from core.data.gen.basic import generate_basic_data
from core.data.gen.demo import (
    generate_demo_data_no_analytics,
    generate_demo_data_with_analytics,
)
from core.data.gen.product_receipts import product_receipts_data
