"""
A module where constants used throughout the 
project should be defined
"""


MAX_EMAIL_LENGTH = 256
MAX_NAME_FIELD_LENGTH = 256
MAX_ORG_ENTITY_NAME_LENGTH = 512

MAX_UUID_LENGTH = 30

UNAMBIGUOUS_CHARS = [
    "B",
    "C",
    "D",
    "F",
    "G",
    "H",
    "J",
    "K",
    "M",
    "P",
    "R",
    "T",
    "V",
    "W",
    "X",
    "Y",
    "2",
    "3",
    "4",
    "6",
    "7",
    "8",
    "9",
]
